@@ -304,6 +304,9 @@ app_get_platform_data (void)
   if ((startup_id = g_getenv ("DESKTOP_STARTUP_ID")))
     g_variant_builder_add (&builder, "{sv}", "desktop-startup-id", g_variant_new_string (startup_id));
 
+  if ((startup_id = g_getenv ("XDG_ACTIVATION_TOKEN")))
+    g_variant_builder_add (&builder, "{sv}", "activation-token", g_variant_new_string (startup_id));
+
   return g_variant_builder_end (&builder);
 }
 
@@ -467,7 +470,7 @@ main (int argc, char **argv)
   if (g_str_equal (argv[1], "list-actions"))
     return app_list_actions (argv + 2);
 
-  g_printerr (_("unrecognised command: %s\n\n"), argv[1]);
+  g_printerr (_("unrecognized command: %s\n\n"), argv[1]);
 
   return app_help (FALSE, NULL);
 }
