@@ -6,6 +6,8 @@
  * Copyright 2001 Hans Breuer
  * Copyright 2004 Tor Lillqvist
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -54,13 +56,16 @@
 
 struct _GDir
 {
+  gatomicrefcount ref_count;
 #ifdef G_OS_WIN32
   _WDIR *wdirp;
 #else
   DIR *dirp;
 #endif
 #ifdef G_OS_WIN32
-  gchar utf8_buf[FILENAME_MAX*4];
+  /* maximum encoding of FILENAME_MAX UTF-8 characters, plus a nul terminator
+   * (FILENAME_MAX is not guaranteed to include one) */
+  gchar utf8_buf[FILENAME_MAX*4 + 1];
 #endif
 };
 
@@ -85,10 +90,13 @@ GDir *
 g_dir_open_with_errno (const gchar *path,
                        guint        flags)
 {
-  GDir dir;
 #ifdef G_OS_WIN32
+  GDir *dir;
+  _WDIR *wdirp;
   gint saved_errno;
   wchar_t *wpath;
+#else
+  DIR *dirp;
 #endif
 
   g_return_val_if_fail (path != NULL, NULL);
@@ -98,25 +106,31 @@ g_dir_open_with_errno (const gchar *path,
 
   g_return_val_if_fail (wpath != NULL, NULL);
 
-  dir.wdirp = _wopendir (wpath);
+  wdirp = _wopendir (wpath);
   saved_errno = errno;
   g_free (wpath);
   errno = saved_errno;
 
-  if (dir.wdirp == NULL)
+  if (wdirp == NULL)
     return NULL;
+
+  dir = g_new0 (GDir, 1);
+  g_atomic_ref_count_init (&dir->ref_count);
+  dir->wdirp = wdirp;
+
+  return g_steal_pointer (&dir);
 #else
-  dir.dirp = opendir (path);
+  dirp = opendir (path);
 
-  if (dir.dirp == NULL)
+  if (dirp == NULL)
     return NULL;
-#endif
 
-  return g_memdup2 (&dir, sizeof dir);
+  return g_dir_new_from_dirp (dirp);
+#endif
 }
 
 /**
- * g_dir_open:
+ * g_dir_open: (constructor)
  * @path: the path to the directory you are interested in. On Unix
  *         in the on-disk encoding. On Windows in UTF-8
  * @flags: Currently must be set to 0. Reserved for future use.
@@ -128,7 +142,7 @@ g_dir_open_with_errno (const gchar *path,
  * directory can then be retrieved using g_dir_read_name().  Note
  * that the ordering is not defined.
  *
- * Returns: a newly allocated #GDir on success, %NULL on failure.
+ * Returns: (transfer full): a newly allocated #GDir on success, %NULL on failure.
  *   If non-%NULL, you must free the result with g_dir_close()
  *   when you are finished with it.
  **/
@@ -183,7 +197,8 @@ g_dir_new_from_dirp (gpointer dirp)
 
   g_return_val_if_fail (dirp != NULL, NULL);
 
-  dir = g_new (GDir, 1);
+  dir = g_new0 (GDir, 1);
+  g_atomic_ref_count_init (&dir->ref_count);
   dir->dirp = dirp;
 
   return dir;
@@ -283,23 +298,84 @@ g_dir_rewind (GDir *dir)
 #endif
 }
 
+static void
+g_dir_actually_close (GDir *dir)
+{
+#ifdef G_OS_WIN32
+  g_clear_pointer (&dir->wdirp, _wclosedir);
+#else
+  g_clear_pointer (&dir->dirp, closedir);
+#endif
+}
+
 /**
  * g_dir_close:
- * @dir: a #GDir* created by g_dir_open()
+ * @dir: (transfer full): a #GDir* created by g_dir_open()
  *
- * Closes the directory and deallocates all related resources.
+ * Closes the directory immediately and decrements the reference count.
+ *
+ * Once the reference count reaches zero, the `GDir` structure itself will be
+ * freed. Prior to GLib 2.80, `GDir` was not reference counted.
+ *
+ * It is an error to call any of the `GDir` methods other than
+ * [method@GLib.Dir.ref] and [method@GLib.Dir.unref] on a `GDir` after calling
+ * [method@GLib.Dir.close] on it.
  **/
 void
 g_dir_close (GDir *dir)
 {
   g_return_if_fail (dir != NULL);
 
-#ifdef G_OS_WIN32
-  _wclosedir (dir->wdirp);
-#else
-  closedir (dir->dirp);
-#endif
-  g_free (dir);
+  g_dir_actually_close (dir);
+  g_dir_unref (dir);
+}
+
+/**
+ * g_dir_ref:
+ * @dir: (transfer none): a `GDir`
+ *
+ * Increment the reference count of `dir`.
+ *
+ * Returns: (transfer full): the same pointer as `dir`
+ * Since: 2.80
+ */
+GDir *
+g_dir_ref (GDir *dir)
+{
+  g_return_val_if_fail (dir != NULL, NULL);
+
+  g_atomic_ref_count_inc (&dir->ref_count);
+  return dir;
+}
+
+/**
+ * g_dir_unref:
+ * @dir: (transfer full): a `GDir`
+ *
+ * Decrements the reference count of `dir`.
+ *
+ * Once the reference count reaches zero, the directory will be closed and all
+ * resources associated with it will be freed. If [method@GLib.Dir.close] is
+ * called when the reference count is greater than zero, the directory is closed
+ * but the `GDir` structure will not be freed until its reference count reaches
+ * zero.
+ *
+ * It is an error to call any of the `GDir` methods other than
+ * [method@GLib.Dir.ref] and [method@GLib.Dir.unref] on a `GDir` after calling
+ * [method@GLib.Dir.close] on it.
+ *
+ * Since: 2.80
+ */
+void
+g_dir_unref (GDir *dir)
+{
+  g_return_if_fail (dir != NULL);
+
+  if (g_atomic_ref_count_dec (&dir->ref_count))
+    {
+      g_dir_actually_close (dir);
+      g_free (dir);
+    }
 }
 
 #ifdef G_OS_WIN32
